﻿#region using

using System;
using System.Data;
using System.Text.RegularExpressions;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using BlogEngine.Core;
using BlogEngine.Core.Web.Controls;
#endregion

/// <summary>
/// Converts KeyWords to inner Link in the comments.
/// 由幻想曲KeyTransfor插件代码修改
/// 感谢博易开源，感谢幻想曲提供的插件
/// </summary>
[Extension("Converts KeyWords to inner Link in the comments", "1.0", "<a href=\"http://wikistory.sokeys.com\" target=\"_blank\">doodc</a>")]
public class KeywordsToLink
{
    static protected ExtensionSettings _settings = null;
    private static StateList<LinkRelating> _sl = null;
    public static readonly object Syncobj = new object();
    private static Regex regex = new Regex("<\\/*[^<>]*>", RegexOptions.Compiled | RegexOptions.IgnoreCase);

    static KeywordsToLink()
    {
        ExtensionSettings settings = new ExtensionSettings("KeywordsToLink");

        settings.AddParameter("Keyword", "key word", 20, true);
        settings.AddParameter("LinkUrl", "URL with http://", 150, true);

        settings.Help = "Converts KeyWords to inner Link in the comments.";
        settings.AddValues(new string[] { "BlogYi", "http://www.blogyi.net" });
        settings.AddValues(new string[] { "博易", "http://www.blogyi.net" });
        settings.AddValues(new string[] { "doodc", "http://wikistory.sokeys.com" });
        //------------------------------------------------------
        ExtensionManager.ImportSettings(settings);
        _settings = ExtensionManager.GetSettings("KeywordsToLink");
        lock (Syncobj)
        {
            if (_sl == null)
            {
                lock (Syncobj)
                {
                    LoadKeyword();
                }
            }
        }
    }

    public KeywordsToLink()
    {
        Post.Serving += new EventHandler<ServingEventArgs>(Post_Serving);
        Post.Saved += new EventHandler<SavedEventArgs>(Post_Saved);
    }

    static void LoadKeyword()
    {
        _sl = new StateList<LinkRelating>();
        LinkRelating lnk;
        DataTable table = _settings.GetDataTable();

        foreach (Post p in Post.Posts.FindAll(delegate(Post post)
        {
            return post.IsPublished == true && post.IsVisible == true;
        }))
        {
            foreach (DataRow row in table.Rows)
            {
                if (String.IsNullOrEmpty(row["Keyword"].ToString()))
                    continue;
                if (regex.IsMatch(row["Keyword"].ToString()))
                    continue;
                if (p.Content.Contains(row["Keyword"].ToString()))
                {
                    lnk = new LinkRelating(row["Keyword"].ToString(), String.Format(@"<a href=""{0}"" target=""_blank"" title=""查看关于{1}"">{1}</a>", row["LinkUrl"].ToString(), row["Keyword"].ToString()));
                    if (!_sl.Contains(lnk))
                        _sl.Add(lnk);

                }
                else
                {
                    continue;
                }
            }
        }
        _sl.TrimExcess();
    }

    private string myReplace(string srcStr, string oldStr, string newStr)
    {
        string strSl, strRl;
        strSl = srcStr.ToLower();
        strRl = oldStr.ToLower();
        int start = strSl.IndexOf(strRl);
        if (start != -1)
        {
            srcStr = srcStr.Substring(0, start) + newStr
            + myReplace(srcStr.Substring(start + oldStr.Length), oldStr, newStr);
        }
        return srcStr;
    }


    void Post_Saved(object sender, SavedEventArgs e)
    {
        LoadKeyword();
    }

    void Post_Serving(object sender, ServingEventArgs e)
    {
        if (e.Location == ServingLocation.SinglePost)
        {
            foreach (LinkRelating lnk in _sl)
            {
                if (regex.IsMatch(lnk.Key))
                    continue;
                string NewBody = e.Body;
                int NewLostNum = 0;
                foreach (Match match in regex.Matches(e.Body))
                {
                    e.Body = e.Body.Replace(match.Value, "NewLostHtml" + NewLostNum + "NewLostHtml");
                    NewLostNum = NewLostNum + 1;
                }
                e.Body = myReplace(e.Body, lnk.Key, lnk.Href);
                NewLostNum = 0;
                foreach (Match match in regex.Matches(NewBody))
                {
                    e.Body = e.Body.Replace("NewLostHtml" + NewLostNum + "NewLostHtml", match.Value);
                    NewLostNum = NewLostNum + 1;
                }
            }
        }
    }

    public struct LinkRelating
    {
        private string _key;

        public string Key
        {
            get { return _key; }
            set { _key = value; }
        }
        private string _href;
        public string Href
        {
            get { return _href; }
            set { _href = value; }
        }

        public LinkRelating(string key, string href)
        {
            _href = href;
            _key = key;
        }
    }
}
